#include <realcnvt.h>
#include "bsreal.h"

#define CR "\r\n"
#define LINE cout << CR; for(i=0;i<80;i++) putch('-');
#define bsreal bassngl
#define ulong unsigned long

#define NA  2

#define GOOD 1
#define YES 1
#define ON  1

#define NOGOOD 0
#define NO  0
#define OFF 0

#define DELETED 1
#define NOMSG   0

#define MULTI_NODE YES

// FOR EXIT_ERR() IN MSGCLS.CPP
//
#define LOCK_ERROR      1
#define WRHDR_ERROR     2
#define SEEK_ERROR      3
#define READ_ERROR      4
#define MEM_ERROR       5

#define IBM_TEXT        1
#define PCB_TEXT        0

#ifndef MSGBASE 
#define MSGBASE

//===================[ MESSAGE BASE STRUCTURES ]====================
//
struct MsgBaseHdr {     // Main message base header
        bsreal  high_msg;
        bsreal  low_msg;
        bsreal  act_msgs;
        bsreal  sys_callers;
        char    locked[6];
        char    reserved[106];
};

struct IMsgBaseHdr{     // Individual Message base header
        char    msg_status;
        bsreal  msg_nmbr;
        bsreal  ref_nmbr;
        char    num_blks;
        char    entry_date[8];
        char    entry_time[5];
        char    msg_to[25];
        bsreal  reply_date;
        char    reply_time[5];
        char    R_reply;
        char    msg_from[25];
        char    msg_subject[25];
        char    msg_password[12];
        char    active_status;
        char    E_echo;
        char    reserved[6];
};

//==================[ MAIN MESSAGE BASE HEADER CLASS ]====================
//
class MBH {
        MsgBaseHdr  MBHdata;
        char    filename[80];  // Filename of message base.
        int     fhandle;       // Read file handle.
        int     whandle;       // Write file handle.
        int     network_flg;   // Are we running multi-node?
        int     MBH_flg;       // Has msg base hdr been saved?
public:
        MBH     (char *,int nwf=NO);
                        // Constructor must be supplied filename
                        // of msg base.  By default is single node.
        ~MBH (void);    // Destructor (frees memory).
        char    *get_filename(void) {return (filename);}
        void    open_lock_mbh(int seconds=30);
                        // Write the message base header. Locks
                        // set to 30 second retry by default.
        void    save_mbh(void);
        void    close_mbh(void);// Close and unlock message file
        int     get_msg_whandle(void) {return(whandle);}
        int     get_msg_fhandle(void) {return(fhandle);}
        ulong   get_high_msg(void);
        ulong   get_low_msg(void);
        ulong   get_act_msgs(void);
        ulong   get_sys_callers(void);
        int     get_network_flg(void);
        int     get_MBH_flg(void){return MBH_flg;}
        int     get_locked(void);
        int     wait_get_locked(int);
        //--------------------------------------
        ulong   set_high_msg(double);
        ulong   set_high_msg(bsreal);
        ulong   set_low_msg(double);
        ulong   set_low_msg(bsreal);
        ulong   set_act_msgs(double);
        ulong   set_act_msgs(bsreal);
        ulong   set_sys_callers(double);
        ulong   set_sys_callers(bsreal);
        int     wait_set_lock(int);
        int     set_lock(void);
        int     remove_lock(void);
};

//===================[ MESSAGE BASE INDEX STUFF ]=====================
//
class msg_index:public MBH{
        int     idx_handle;
        char    ndx_filename[80];
        ulong   lowmsg;
        ulong   highmsg;
        ulong   u_offset;
        bsreal  b_offset;
public:
        msg_index(char*,int NWF);
        ~msg_index(void);
        ulong   get_msgidx_offset(ulong);
        ulong   set_msgidx_offset(ulong,ulong);
        char    *get_ndx_filename(void){return ndx_filename;};
};

//=================[ INDIVIDUAL MESSAGE HEADER ]======================
//
class IMBH {
        IMsgBaseHdr  IMBHdata;
        char    *text;
        int     text_flag; // We'll set this if we convert all
                           //  to 0d 0a because we'll need to
                           // change it back if we are writing
                           // new message to message base.
        char    tagline[80];
        ulong   node_msg;
public:
        IMBH (void){text=NULL;} // Constructor for message write.
        IMBH (int,ulong);// Constructor; supply message handle and offset in msg base
        ~IMBH(void);     // Destructor

        // The following 3 functions manipulate the data prior to returning it
        char    *status_type(void);      
        char    *message_read(void);
        char    *message_refer(void);

        void    save_message(msg_index&);

        //-----------------------------
        char    *get_msg_text(int mode=IBM_TEXT);
        char    *IBM_to_PCB(char*);
        int     get_num_blocks(void);
        char    get_msg_status(void);
        ulong   get_msg_nmbr(void);
        ulong   get_ref_nmbr(void);
        char    *get_entry_date(void);
        char    *get_entry_time(void);
        char    *get_msg_to(void);
        char    *get_reply_date(void);
        char    *get_reply_time(void);
        int     get_R_reply(void);
        char    *get_msg_from(void);
        char    *get_msg_subject(void);
        char    *get_msg_password(void);
        int     get_active_status(void);
        int     get_E_echo(void);
        //------------------------------
		int     set_msg_status(char);
        ulong   set_msg_nmbr(ulong);
        ulong   set_msg_nmbr(bsreal);
        ulong   set_ref_nmbr(ulong);
        ulong   set_ref_nmbr(bsreal);
        int     set_num_blks(int);
        char    *set_entry_date(char *);
        char    *set_entry_time(char *);
        char    *set_msg_to(char *);
        void    set_reply_date(char *);
        void    set_reply_date(ulong);
        void    set_reply_date(bsreal);
        char    set_reply_time(char *);
        int     set_R_reply(int);
		char    *set_msg_from(char *);
		char    *set_msg_subject(char *);
		char    *set_msg_password(char *);
		int     set_active_status(int);
        int     set_E_echo(int);
};

#endif
