#include <conio.h>

#include <string.h>
#include <stdlib.h>
#include <io.h>
#include <iostream.h>
#include <alloc.h>
#include "msgcls.h"
#include "bsreal.h"
#include <mem.h>

extern exit_err(int);  // ROUTINE IS IN MSGCLS.CPP

//===================[ CONSTRUCTOR/DESTRUCTOR FOR IMBH CLASS ]===============
//===================[ CONSTRUCTOR/DESTRUCTOR FOR IMBH CLASS ]===============
//===================[ CONSTRUCTOR/DESTRUCTOR FOR IMBH CLASS ]===============
//
// CONSTRUCTOR FOR INDIVIDUAL MESSAGE CLASS (reads in message header and text)
// Send constructor the (open) file handle for the applicable message file
// and the messages offset in the message file.
//
IMBH::IMBH(int handle,ulong offset){
        text=NULL;
        text_flag=PCB_TEXT;     // Default set to PCB mode (no 0d 0a in text)

        if (offset!=0){
                if((lseek(handle,offset,SEEK_SET))==-1) exit_err(SEEK_ERROR);
                if((read(handle,&IMBHdata,sizeof(IMsgBaseHdr)))!=sizeof(IMsgBaseHdr)) exit_err(READ_ERROR);
                int blocks=((int)IMBHdata.num_blks*128);
                if(blocks==0) return;   // Just in case....
                        // Notice that we are allocating 128 bytes more then required.
                        // This is so we can convert the message base to TEXT
                        // (i.e.,  to 0d 0a)
                if((text=(char*) malloc(blocks))==NULL) exit_err(MEM_ERROR);
                memset(text,'\0',blocks);       // We are going to count on these NULLs to
                                                // Help us find the end of the text message.
                if((read(handle,text,blocks-128))!=blocks-128)  // We have 128 more bytes
                        exit_err(READ_ERROR);                   // then we need.
                }
}

IMBH::~IMBH(void){
        if(text!=NULL);
                free(text);
}


//================[ GET MEMBER LIST DATA FROM CLASS IMBH ]==================
//================[ GET MEMBER LIST DATA FROM CLASS IMBH ]==================
//================[ GET MEMBER LIST DATA FROM CLASS IMBH ]==================
//
// GET ECHO MAIL STATUS
//
int IMBH::get_E_echo(){ return((int) (IMBHdata.E_echo=='E')?YES:NO);}

// GET ACTIVE STATUS
//
int IMBH::get_active_status(){ return((int) (IMBHdata.active_status==225)?YES:NO);}

// GET MESSAGE PASSWORD
//
char* IMBH::get_msg_password(){
        static char mp[13];
        memset(mp,'\0',13);
        memmove(mp,IMBHdata.msg_password,12);
        return(mp);
}
// GET MESSAGE SUBJECT
//
char* IMBH::get_msg_subject(){
        static char ms[26];
        memset(ms,'\0',26);
        memmove(ms,IMBHdata.msg_subject,25);
        return(ms);
}
// GET MESSAGE FROM
//
char* IMBH::get_msg_from(){
        static char mf[26];
        memset(mf,'\0',26);
        memmove(mf,IMBHdata.msg_from,25);
        return(mf);
}
// IF THERE IS A REPLY
//
int IMBH::get_R_reply(){ return ((int)(IMBHdata.R_reply=='R')?YES:NO);}

// GET REPLY_TIME
//
char* IMBH::get_reply_time(){
        static char *rt="HH:MM";
        memmove(rt,IMBHdata.reply_time,5);
        return (rt);
}

// GET REPLY DATE (BSREAL VALUE)
//
char* IMBH::get_reply_date(){
                static char holdtank[8],datefld[]={"MM-DD-YY"};
                BSreal rd(IMBHdata.reply_date);
                ltoa(rd.get_long_value(),holdtank,10);
                if((memcmp(holdtank,"0",1))==0) return ("NO REPLY DATE");
                memmove(&datefld[6],holdtank,2);
                memmove(datefld,&holdtank[2],2);
                memmove(&datefld[3],&holdtank[4],2);
                return (datefld);
}

// GET MESSAGE TO
//
char* IMBH::get_msg_to(){
        static char mt[26];
        memset(mt,'\0',26);
        memmove(mt,IMBHdata.msg_to,25);
        return (mt);
}

// GET ENTRY TIME OF MESSAGE
//
char* IMBH::get_entry_time(){
        static char *tm="HH:MM";
        memmove(tm,IMBHdata.entry_time,5);
        return (tm);
}

// GET ENTRY DATE OF MESSAGE
//
char* IMBH::get_entry_date(){
        static char *dt="MM-DD-YY";
        memmove(dt,IMBHdata.entry_date,8);
        return (dt);
}

// GET NUMBER OF BLOCKS IN MESSAGE
//
int IMBH::get_num_blocks(){ return ((int)IMBHdata.num_blks);}

// GET REFERENCE NUMBER
//
ulong IMBH::get_ref_nmbr(){
        BSreal rn(IMBHdata.ref_nmbr);
        return (rn.get_long_value());
}

// GET MESSAGE NUMBER
//
ulong IMBH::get_msg_nmbr(){
        BSreal mn(IMBHdata.msg_nmbr);
        return(mn.get_long_value());
}

// GET MESSAGE STATUS
//
char IMBH::get_msg_status(){    return IMBHdata.msg_status;     }

//======[ MISC MEMBER FUNCTIONS TO MANIPULATE DATA IN MEMBER LIST ]========
//======[ MISC MEMBER FUNCTIONS TO MANIPULATE DATA IN MEMBER LIST ]========
//======[ MISC MEMBER FUNCTIONS TO MANIPULATE DATA IN MEMBER LIST ]========
//
// Return a character string representing message reference number or NONE.
//
char* IMBH::message_refer(){
        static char refer[14];
        ulong rn=get_ref_nmbr();
        if(rn==0)
                return("NONE");
        ltoa(get_ref_nmbr(),refer,10);
        return(refer);
}

// Return character string in MM-DD-YY (HH:MM) format for message read data/time
//
char* IMBH::message_read(){
        char status=IMBHdata.msg_status;
        static char rstr[]={"MM-DD-YY (HH:MM)"};
        if((status=='+')||(status=='-')||(status=='`')){
                memmove(rstr,get_reply_date(),8);
                memmove(&rstr[10],get_reply_time(),5);
                return(rstr);
        } else
                return ("NO");
}

// Returns character string representing the status of the message.
//
char* IMBH::status_type(){
        char status=IMBHdata.msg_status;
        switch (status){
                case '*':
                case '+': return "RECEIVER ONLY";
                case ' ':
                case '-': return "PUBLIC MESSAGE";
                case '~':
                case '`': return "COMMENT";
                case '%':
                case '^': return "SENDER PASSWORD";
                case '!':
                case '#':
                case '$': return "GROUP PASSWORD";
        default: return "UNKNOWN";
        }
}

// Return text string in PCBoard or IBM text format (default is IBM)
//
char* IMBH::get_msg_text(int mode){
        char *ptr,*tmp,*tmpptr;
        tmp=NULL;
        if (text==NULL) return "";      // In case we're returning to cout
        if(mode==PCB_TEXT) return text;
        text_flag=IBM_TEXT;
        int bytes=strlen(text);
        if((tmp=(char*)malloc(bytes+1))==NULL){ // let's drag a null with us
                cout << "\nError encountered trying to allocate memory for\nconverted text!!!\a";
                exit(1);}
        memmove(tmp,text,bytes+1);
        ptr=text;
        tmpptr=tmp;             // We can't free tmp if we move it so we use a pointer
        while (*tmpptr!=NULL){
                *ptr=*tmpptr;
                if(*tmpptr==''){
                        *ptr='\15';
                        ptr++;
                        *ptr='\12';}
                ptr++;
                tmpptr++;}
        if(tmp!=NULL)
                free(tmp);
        return(text);
}

// Returns pointer to a string converted to PCBoard format.  Send this 
// function a text string (with CR/LF).  Sets member data so no copying
// is required.
//
char* IMBH::IBM_to_PCB(char *str){
        char *ptr,*tmp,*tmpptr;
        tmp=NULL;
                ptr=str;
        if ((str==NULL)||(*str=='\0')) return "";       // In case we're returning to cout
                int bytes=strlen(ptr);
                if((tmp=(char*)malloc(bytes+1))==NULL){ // let's drag a null with us
                cout << "\nError encountered trying to allocate memory for\nconverted text!!!\a";
                exit(1);}
        memset(tmp,'\0',bytes+1);
                memmove(tmp,ptr,bytes+1);
        tmpptr=tmp;             // We can't free tmp if we move it so we use a pointer
        while (*tmpptr!=NULL){
                *ptr=*tmpptr;
                if(*tmpptr=='\r'){
                        *ptr='\';
                        tmpptr++;}
                ptr++;
                tmpptr++;}
        *ptr='\0';
                if((ptr=strstr(str,"\x1A"))!=NULL) *ptr='\0';
        if(tmp!=NULL)
                                free(tmp);      // We're done with tmp.

                unsigned int blocks=strlen(str)/128+1;
                bytes=blocks*128;
                if((text=(char*)malloc(bytes))==NULL){
                        cout << CR"Error encountered allocating memory for message text (1)";
                        cout << CR"Process aborted...";
                        exit(1);
                }
                IMBHdata.num_blks=(char)blocks+1;

        memset(text,' ',((int)IMBHdata.num_blks-1)*128);
                memmove(text,str,strlen(str)-1);  // Don't want to copy NULL
                return(text);
}

//:::::::::::::::[ SET INDIVIDUAL MESSAGE HEADER DATA ]:::::::::::::::::
//:::::::::::::::[ SET INDIVIDUAL MESSAGE HEADER DATA ]:::::::::::::::::
//:::::::::::::::[ SET INDIVIDUAL MESSAGE HEADER DATA ]:::::::::::::::::
//

// Set data using IEEE

ulong IMBH::set_msg_nmbr(ulong mn){
        BSreal msg(mn);
        msg.copy_BSreal_to(IMBHdata.msg_nmbr);
        return(msg.get_long_value());
}

ulong IMBH::set_ref_nmbr(ulong rn){
        BSreal msg(rn);
        msg.copy_BSreal_to(IMBHdata.ref_nmbr);
        return(msg.get_long_value());
}


// Set data using bsreal

ulong IMBH::set_msg_nmbr(bsreal mn){
        BSreal msg(mn);
        msg.copy_BSreal_to(IMBHdata.msg_nmbr);
        return(msg.get_long_value());
}


ulong IMBH::set_ref_nmbr(bsreal rn){
        BSreal msg(rn);
        msg.copy_BSreal_to(IMBHdata.ref_nmbr);
        return(msg.get_long_value());
}


// SET MESSAGE STATUS WITH CHAR
//
int IMBH::set_msg_status(char status){
        IMBHdata.msg_status=status;
        return((int)IMBHdata.msg_status);
}

// SET ENTRY DATE IN MM-DD-YY FORMAT
//
char *IMBH::set_entry_date(char *entrydate){
        memmove(IMBHdata.entry_date,entrydate,8);
        return(IMBHdata.entry_date);
}

// SET ENTRY TIME IN HH:MM FORMAT
//
char *IMBH::set_entry_time(char *entrytime){
        memmove(IMBHdata.entry_time,entrytime,5);
        return(IMBHdata.entry_time);
}

char *IMBH::set_msg_to(char *msgto){
        memmove(IMBHdata.msg_to,msgto,strlen(msgto));
        return(IMBHdata.msg_to);
}

char *IMBH::set_msg_from(char *msgfrom){
        memmove(IMBHdata.msg_from,msgfrom,strlen(msgfrom));
        return(IMBHdata.msg_from);
}

char *IMBH::set_msg_subject(char *msgsubj){
        memmove(IMBHdata.msg_subject,msgsubj,strlen(msgsubj));
        return(IMBHdata.msg_subject);
}

int IMBH::set_active_status(int status){
        IMBHdata.active_status=status;
        return((int)IMBHdata.active_status);
}


void IMBH::save_message(msg_index& Data){
        ulong wmh=Data.get_msg_whandle();
        ulong fl=filelength(wmh);       // get length of MSGS file
        ulong status=lseek(wmh,fl,SEEK_SET);

        if((write(wmh,&IMBHdata,sizeof(IMsgBaseHdr)))!=sizeof(IMsgBaseHdr)){
                Data.remove_lock();     // Remove locks before we exit
                cout << CR"Error encountered writing message header";
                exit(1);}

        ulong blocksize=(IMBHdata.num_blks-1)*128;
        ulong msgnumber=get_msg_nmbr();

        if((write(wmh,text,blocksize))!=blocksize){
                Data.remove_lock();             // Remove locks before we exit
                cout << CR"Error encountered writing message text";
                exit(1);}

        if((Data.set_msgidx_offset(msgnumber,fl))==-1){ // Write failed if -1
                Data.remove_lock();
                exit(1);
        }
}









