#include <io.h>
#include <bios.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <conio.h>
#include <iostream.h>
#include <string.h>

#include "cnmcls.h"
#include "msgcls.h"
#include "tools.h"
#include "bsreal.h"

char cn[80];    // CNAMES.@@@ LOCATION
int NWF=0;      // Multi-node flag (default is single node);

//======================[ PCBOARD.DAT FUNCTIONS ]======================
//
// RETURNS DATA FROM PCBOARD.DAT SPECIFIED BY 'field'.  CR IS REMOVED FROM LINE
//
char *get_pcboard_dat(int field){
        char *ptr;
        int cntr=0;
        static char buffer[80];
        // NOTE: We don't want to mix-match file access activity so we run this
        //       before we do anything else to retrieve location of CNAMES.@@@
        //
        FILE *fn;
        if((fn=fopen("PCBOARD.DAT","rt"))==NULL){
                cout << CR"Could not find PCBOARD.DAT file!!!\a"CR;
                exit(1);}
        while((fgets(buffer,80,fn))!=NULL)
                if(++cntr==field) break;
        fclose(fn);
        if((ptr=strstr(buffer,"\n"))!=NULL) *ptr='\0';
        return(buffer);
}

//========================[ MESSAGE FUNCTIONS ]========================
//
// APPENDS MESSAGE IN 'IMBH' OBJECT TO CONFERENCE SPECIFIED BY 'confnum'
//
void append_message(IMBH& message, ulong confnum){

        CONFDATA confdata(cn);                // Open CNAMES.@@@ (cn=conference name
                                                                                  // which is a global variable).
        confdata.get_conf_data(confnum);      // Read in conference #'confnmbr' data
        char *msgs=confdata.get_msg_file();   // Get MSGS location
        msg_index Data(msgs,NWF);             // Open Message Base and Index with NetWork Flag

        ulong hm=Data.get_high_msg();   // Get high message for this base
        ulong am=Data.get_act_msgs();   // Get active messages for this base

        BSreal(hm);
        BSreal(am);
        hm++;   // Increment High Message
        am++;   // Increment Active Messages

        // NOTE: All of the remaining functions will remove locks if function
        //       contains an error exit routine.
        //
        Data.open_lock_mbh();   // Open message base for writing.

        message.set_msg_nmbr(hm);       // Set high message in header
        message.set_ref_nmbr((ulong)0); // Set reference to 0
        message.save_message(Data); // save this object to MSGS and MSGS.NDX
                                    // this routine will append the message.
        Data.set_high_msg(hm);          // Set message base high message.
        Data.set_act_msgs(am);          // Set message base active messages.

        Data.save_mbh();                // Save message base header with new high/active message
        Data.close_mbh();               // Close file and remove locks
}

//======================[ CHECK STATUS OF FLOPPY DRIVE ]====================
//
int floppy_status(int drv){
        int cntr=0;
        union REGS inregs;
        for(;;){
                // Verify disk sectors (no data is transferred)
                //
                inregs.x.ax = 0x401;  // (H)func 4  (L)# sectors=1
                inregs.x.cx = 0x01;   // (H)track 0 (L)sector 0
                inregs.x.dx = drv;    // (H)head 0  (L)drive
                int86(0x13, &inregs, &inregs);

                if(!inregs.h.ah) break;  // if successful return 0
                if(cntr++>0) break;  // one retry after failure

                // reset disk on verify error (could be from previous
                // error).  Retry one more time after reset.
                //
                inregs.x.ax = 0x00;  // (H)func 0
                inregs.x.dx = drv;   // (L)drive
                int86(0x13, &inregs, &inregs);
        }
        return(!inregs.h.ah);
}


//====================[ BIT MANIPULATION FUNCTIONS ]===================
//
// TOGGLE BIT IN CHAR STRING  
void toggle_bit_byte(char *bitstr, int byte, int bit){
        for(int i=0;i<byte;i++) bitstr++;
        i=1<<bit;
        *bitstr=*bitstr^i;
}
// GET BIT IN CHAR STRING  
int get_bit_byte(char *bitstr, int byte, int bit){
        for(int i=0;i<byte;i++) bitstr++;
        i=1<<bit;
        return( (*bitstr&i)==0 ? 0:1 );
}
// SET BIT IN CHAR STRING ON OR OFF  
void set_bit_byte(char *bitstr, int byte, int bit, int mode){
        if((get_bit_byte(bitstr,byte,bit)!=mode))
                toggle_bit_byte(bitstr,byte,bit);
}

//========================[ STRING FUNCTIONS ]=============================
//
// TRIM TRAILING SPACES FROM STRING
//
char *trim(char *str){
        int len=strlen(str);
        char *ptr;
        ptr=str;
        ptr+=len-1;
        while(*ptr==' ') ptr--;
        ptr++;
        *ptr='\0';
        return(str);
}

//========================[ DATE/TIME ROUTINES ]===========================
//
// MILITARY TIME RETURNED IN HH:MM FORMAT
//
char * timestr(void) {
        static char Time[6];
        union REGS ireg;
        ireg.h.ah=0x2c;
        intdos(&ireg,&ireg);
        sprintf(Time,"%02d:%02d",ireg.h.ch,ireg.h.cl);
        return(Time);
}

// DATE RETURNED IN MM-DD-YY FORMAT
//
char * datestr(void) {
        static char Date[9];
        union REGS ireg;
        ireg.h.ah=0x2a;
        intdos(&ireg,&ireg);
        sprintf(Date,"%02d-%02d-%02d",ireg.h.dh,ireg.h.dl,ireg.x.cx-1900);
        return(Date);
}

// DATE RETURNED IN UNSIGNED LONG YYMMDD FORMAT
//
unsigned long dateval(void) {
        static char Date[9];
        union REGS ireg;
        ireg.h.ah=0x2a;
        intdos(&ireg,&ireg);
        sprintf(Date,"%02d%02d%02d",ireg.x.cx-1900,ireg.h.dl,ireg.h.dh,ireg.h.dl);
        return((unsigned long) atol(Date));

}

/*=============================[ PRINTER ]==================================*/
//
// CHECK STATUS OF PRINTER PORT (RETURNS -1 AND EXECUTES PRNERR.BAT ON ERROR)
//
int prtck(int port){
   #define STATUS  2    /* printer status command */
   int status, abyte=0;
   status = biosprint(STATUS, abyte, port);
       if ((status & 0x80)!=0x80){
               clrscr();
                printf(CR CR"Device ERROR!  Process aborted...\a");
                if (status & 0x01)
                        printf(CR"(0x01)  Device time out.\n");
                if (status & 0x08)
                        printf(CR"(0x08)  I/O error.\n");
                if (status & 0x10)
                        printf(CR"(0x10)  Selected.\n");
                if (status & 0x20)
                        printf(CR"(0x20)  Out of paper.\n");
                if (status & 0x40)
                        printf(CR"(0x40)  Acknowledge.\n");
                if((access("PRNERR.BAT",0))==0) system("PRNERR");
                return(-1);
        } else
                return(0);
}

// SEND STRING TO PRINTER
//
void prtstr(int port,char *str){
        while(*str){
                biosprint(0,*str,port);
                str++;}}


// SEND FORM FEED TO PRINTER
//
void eject(int port){biosprint(0,0x0C,port);}


